# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import gtk
import gtk.glade
from os import path

from brisa.config import conf
import brisa

class ConfigGTKUI(object):

    log_level_list = ['NOSET', 'INFO', 'WARNING', 'DEBUG', 'ERROR', 'CRITICAL']
    log_out_list = ['file', 'stdout']
    true_list = ['on', 'yes', '1', 1, True]
    conf_dict = {'brisa': '', 'logging': '', 'audio': '', 'video': '',
                 'image': '', 'youtube': '', 'flickr': '', 'shoutcast': '',
                 'connection': '', 'persistence': '', 'maemo_multimedia': '', 'canola': ''}
    location = path.dirname(__file__)

    def __init__(self):
        gladefile = path.join(self.location, 'config_gtk.glade')
        self.wTree = gtk.glade.XML(gladefile)
        self.window = self.wTree.get_widget('window1')

        self.show_logo()
        if self.window:
            self.window.connect('destroy', gtk.main_quit)

        events = {'on_close_btn_clicked': gtk.main_quit,
                   'on_about_btn_clicked': self.about,
                   'on_save_btn_clicked': self.save,
                   'on_discard_btn_clicked': self.discard,
                   'on_audio_select_clicked': self.select_dialog,
                   'on_image_select_clicked': self.select_dialog,
                   'on_video_select_clicked': self.select_dialog,
                   }

        self.wTree.signal_autoconnect(events)
        self.refresh_data()

    def show_logo(self):
        logo = gtk.gdk.pixbuf_new_from_file(path.join(self.location,
                                                      'brisa.png'))
        logo_handler = self.wTree.get_widget('logo')
        logo_handler.set_from_pixbuf(logo)

    def load_conf(self):
        for section in self.conf_dict:
            self.conf_dict[section] = conf.items(section)

    def refresh_data(self):
        self.load_conf()
        data = self.conf_dict
        for section in data:
            for parameter in data[section]:
                item, value = parameter
                widget_name = '%s_%s' % (section, item)
                widget = self.wTree.get_widget(widget_name)
                if not widget:
                    continue
                elif isinstance(widget, gtk.Entry):
                    widget.set_text(value)
                elif isinstance(widget, gtk.ComboBox):
                    index = 0
                    if widget_name == 'logging_output':
                        index = self.log_out_list.index(value)
                    elif widget_name == 'logging_level':
                        index = self.log_level_list.index(value)
                    widget.set_active(index)
                elif isinstance(widget, gtk.CheckButton):
                    widget.set_active(value.lower() in self.true_list)

    def save(self, widget):
        data = self.conf_dict
        for section in data:
            for parameter in data[section]:
                item = parameter[0]
                widget_name = '%s_%s' % (section, item)
                widget = self.wTree.get_widget(widget_name)
                if not widget:
                    continue
                elif isinstance(widget, gtk.Entry):
                    value = widget.get_text()
                elif isinstance(widget, gtk.ComboBox):
                    index = widget.get_active()
                    if widget_name == 'logging_output':
                        value = self.log_out_list[index]
                    elif widget_name == 'logging_level':
                        value = self.log_level_list[index]
                elif isinstance(widget, gtk.CheckButton):
                    value = widget.get_active()
                    if value:
                        value = 'yes'
                    else:
                        value = 'no'
                conf.set_parameter(section, item, value)

    def about(self, widget):
        window = gtk.AboutDialog()
        window.set_name("BRisa UPnP A/V Framework")
        window.set_version(brisa.__version__)
        window.set_license(''' Copyright (c) <2007> <Brisa Team>
        Permission is hereby granted, free of charge, to any person obtaining a
        copy of this software and associated documentation files (the
        "Software"), to deal in the Software without restriction, including
        without limitation the rights to use, copy, modify, merge, publish,
        distribute, sublicense, and/or sell copies of the Software, and to
        permit persons to whom the Software is furnished
        to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be included
        in all copies or substantial portions of the Software.

        THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
        OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
        MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
        IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
        CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
        TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
        SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
        ''')
        window.set_authors(['Adrian Livio Vasconcelos Guedes',
                            'Ana Julia Oliveira',
                            'Danilo Freire',
                            'Francisco CS Junior',
                            'Jose Luis do Nascimento',
                            'Leandro Melo de Sales'])
        window.set_copyright("\302\251 Copyright 2007-2008 BRisa Team")
        window.set_website("http://brisa.garage.maemo.org/")
        window.connect("response", lambda d, r: d.destroy())
        window.show()

    def select_dialog(self, widget):
        window = gtk.FileChooserDialog(title='BRisa Configuration Tool',
                                action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                         gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        window.set_select_multiple(False)
        if window.run() == gtk.RESPONSE_OK:
            dir_name = window.get_filename()
            widget_name = '%s_dirs' % widget.get_name().split('_')[0]
            widget = self.wTree.get_widget(widget_name)
            widget.set_text('%s:%s' % (widget.get_text(), dir_name))
        window.destroy()

    def discard(self, widget):
        self.refresh_data()

    def show(self):
        gtk.main()

def main():
    window = ConfigGTKUI()
    window.show()

if __name__ == "__main__":
    main()
