# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import os
from urllib import splitquery
from cgi import parse_qs

from brisa import log, config
from brisa.main import ThreadManager
from brisa.services.web_server import StaticFile, WebServer, CustomResource
from brisa.utils.network import get_ip_address


class BrisaConfServer(object):

    static_pages = ['brisa_style.css', 'top_frame.html', 'about.html']
    page_list = {'shoutcast': ['usage'],
                 'image': ['dirs'],
                 'flickr': ['usage', 'username', 'private'],
                 'video': ['dirs'],
                 'audio': ['dirs'],
                 'persistence': ['connection']}

    def __init__(self, port_number):
        self.port = port_number
        self.root = CustomResource()
        self.root.add_static_file('', StaticFile(
                                        os.path.join(os.path.dirname(__file__),
                                                      'brisa.html')))
        self.root.add_static_file('favicon.ico',
                                  StaticFile(os.path.join(
                                                os.path.dirname(__file__),
                                                'favicon.ico')))
        self.load_images()
        self.load_static_pages()
        self.load_dynamic_pages()
        self.load_webserver()

    def load_images(self):
        imgResource = CustomResource()
        self.root.add_resource('images', imgResource)
        for i in os.listdir(os.path.join(os.path.dirname(__file__), 'images')):
            imgResource.add_static_file(i,
                                        StaticFile(os.path.join(
                                                    os.path.dirname(__file__),
                                                    'images/' + i)))

    def load_static_pages(self):
        for i in self.static_pages:
            self.put_static_file(i)

    def load_dynamic_pages(self):
        putChild = self.root.add_static_file
        page_list = self.page_list
        putChild('brisa_main.html', MainPage())
        for key in self.page_list:
            mypage = DynamicPage(key, page_list[key])
            putChild('%s%s' % (key, '.html'), mypage)

    def load_webserver(self):
        self.webserver = WebServer()
        self.webserver.root_dir = self.root
        self.webserver.listen_port = self.port

    def put_static_file(self, file_name):
        pwd = os.path.dirname(__file__)
        self.root.add_static_file(file_name, StaticFile(
                                                os.path.join(pwd, file_name)))

    def run(self):
        log.info('Starting Brisa web configuration interface.')
        self.webserver.start()
        ThreadManager().main_loop()

class DynamicPage(CustomResource):
    page_attr = []
    page_id = ''
    page_name = ''

    def __init__(self, id, attr):
        CustomResource.__init__(self)
        self.page_attr = attr
        self.page_id = id
        self.page_name = id + '.html'
        self.params = None
        self.has_extras = False
        if self.page_id == 'video' or self.page_id == 'audio' or \
                self.page_id == 'image':
            self.has_extras = True

    def getChildWithDefault(self, uri, params):
        self.params = params
        return self

    def render(self, uri, request, response):
        fd = open(os.path.join(os.path.dirname(__file__), self.page_name))
        homepage = fd.read()
        fd.close()
        result = self.get_config_parameters()
        extra = ''
        if self.has_extras:
            dirs = '/'
            parent_dirs = dirs
            if self.params:
                dirs = self.params['dirs']
                if dirs[-1] != '/':
                    parent_dirs = os.path.split(dirs)[0]
                    dirs += '/'
            extra = self.html_convert(self.list_localdirs(dirs), parent_dirs)
            result.append(dirs)
            result.append(extra)
        homepage = homepage % tuple(result)
        return homepage

    def get_config_parameters(self):
        get_parameter = config.get_parameter
        page_id = self.page_id
        return [get_parameter(page_id, i) for i in self.page_attr]

    def list_localdirs(self, root):
        dirs = os.listdir(root)
        result = []
        append = result.append
        for entry in dirs:
            fep = '%s%s' % (root, entry)
            if os.path.isdir(fep):
                append(fep)
        result.sort()
        return result

    def html_convert(self, list, parent_dirs):
        base = '<a href="%s?dirs=%s">\
        <strong>..</strong></a>\n<br>' % (self.page_name, parent_dirs)
        tail = ['<a href="%s?dirs=%s">%s</a>\n<br>' %
                (self.page_name, entry, entry) for entry in list]
        tail = ''.join(tail)
        html_code = '%s%s' % (base, tail)
        return html_code

class MainPage(CustomResource):

    def __init__(self):
        CustomResource.__init__(self)
        self.params = None

    def getChildWithDefault(self, uri, params):
        self.params = params
        return self

    def render(self, uri, request, response):
        if self.params:
            self.update_config(self.params)

        fd = open(os.path.join(os.path.dirname(__file__), 'brisa_main.html'))
        homepage = fd.read()
        fd.close()
        parms = {'version': config.get_parameter('brisa', 'version'),
                 'server_status': self.mediaserver_status(),
                 'persistence_conn': config.get_parameter('persistence',
                                                                'connection'),
                 'audio_usage': config.get_parameter('audio', 'usage'),
                 'audio_dirs': config.get_parameter('audio', 'dirs'),
                 'video_usage': config.get_parameter('video', 'usage'),
                 'video_dirs': config.get_parameter('video', 'dirs'),
                 'image_usage': config.get_parameter('image', 'usage'),
                 'image_dirs': config.get_parameter('image', 'dirs'),
                 'flickr_usage': config.get_parameter('flickr', 'usage'),
                 'flickr_username': config.get_parameter('flickr',
                                                               'username'),
                 'flickr_private': config.get_parameter('flickr',
                                                              'private'),
                 'shoutcast_usage': config.get_parameter('shoutcast',
                                                               'usage')}
        return homepage % parms

    def mediaserver_status(self):
        log.debug('Getting Media Server Status')
        pids = self.mediaserver_pids()
        if pids:
            status = '%s%s' % ('Running on PIDs: ', str(pids))
        else:
            status = 'Stopped'
        log.debug('Media Server Status: %s', status)
        return status

    def stop_mediaserver(self):
        if self.mediaserver_status() != 'Stopped':
            pids = self.mediaserver_pids()
            kill = os.kill
            (kill(int(pid), 15) for pid in pids)

    def start_mediaserver(self):
        if self.mediaserver_status() == "Stopped":
            try:
                pid = os.fork()
                if pid == 0:
                    os.execv('/usr/bin/brisa-server', ['', '--daemon'])
                else:
                    os.wait()
            except OSError, e:
                log.error('Fail on forking process')

    def mediaserver_pids(self):
        pid_list = []
        append = pid_list.append
        for p in os.listdir("/proc"):
            path = "/proc/%s/cmdline" % p
            try:
                cmdline = open(path).read()
            except (IOError, OSError), e:
                continue
            if "brisa-server" in cmdline:
                append(int(p))
        return pid_list

    def update_config(self, result):
        plugin_name = result['plugin_name']
        log.debug('Updating configuration to %s plugin', plugin_name)
        if plugin_name == 'flickr':
            if result.has_key('usage') and result.has_key('username') and \
            result.has_key('private'):
                config.set_parameter('flickr', 'usage', result['usage'])
                config.set_parameter('flickr', 'username', result['username'])
                config.set_parameter('flickr', 'private', result['private'])
        elif plugin_name == 'shoutcast':
            if result.has_key('usage'):
                config.set_parameter('shoutcast', 'usage', result['usage'])
        elif plugin_name == 'sound':
            if result.has_key('dir'):
                config.set_parameter('audio', 'dirs', result['dir'])
        elif plugin_name == 'video':
            if result.has_key('dir'):
                config.set_parameter('video', 'dirs', result['dir'])
        elif plugin_name == 'image':
            if result.has_key('dir'):
                config.set_parameter('image', 'dirs', result['dir'])
        elif plugin_name == 'main':
            if result.has_key('action'):
                if result['action'] == 'start':
                    self.start_mediaserver()
                elif result['action'] == 'stop':
                    self.stop_mediaserver()
        else:
            log.debug('error: %s is not defined', plugin_name)
