# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import sys

from brisa import log
from brisa.control_point.event import EventListenerServer
from brisa.upnp.didl.DIDLLite import Element
from brisa.upnp.message_handle.device_description_object import \
    DeviceDescriptionObject
from brisa.upnp.msearch import MSearch
from brisa.upnp.soap import SOAPProxy
from brisa.upnp.ssdp import SSDPServer
from brisa.utils import network


def get_device_object(location):
    """Make a descripion of an object.

    Call DeviceDescriptionObject to make a descripion of an object
    represented by location"""
    return DeviceDescriptionObject(location).toObject()


def get_device_object_async(location, callback, cargo):
    DeviceDescriptionObject(location).toObject_async(callback, cargo)


def get_service_control_url(service_dict, service_type):
    """Get list of control_url.

    This method returns a list of all control_url service """
    for k, service in service_dict.items():
        service = service_dict[k]
        if service.service_type.startswith(service_type):
            return service.control_url


class ControlPoint(object):
    """This class implements the basic UPnP Control Point functionalities.
    """
    encoding = 'UTF-8'

    def __init__(self):
        """ControlPoint class constructor.

        This constructor receives an object that will be notified according to
        the changes on the list of media servers/renderers found. It
        implements the observer design pattern."""

        self._ssdp_server = SSDPServer("BRisa Control Point", None)
        self._ssdp_server.subscribe("new_device_event", self._new_device_event)
        self._ssdp_server.subscribe("removed_device_event",
                                   self._removed_device_event)

        self._msearch = None
        self._callbacks = {}

        # EventListenerServer takes care of handling the web server
        self._event_listener = EventListenerServer(self)

        # Data for event subscribing
        # event_host is necessary for event subscribing, so it can tell the
        # IP:port of the event listener in subscription, without having to
        # refer to ControlPoint inside service.py (we presume that subscription
        # caller "knows" ControlPoint and Services, so it is in better position
        # of forwarding this info)
        self.event_host = self._event_listener.host()

    def subscribe(self, name, callback):
        self._callbacks.setdefault(name, []).append(callback)

    def unsubscribe(self, name, callback):
        callbacks = self._callbacks.get(name, [])
        if callback in callbacks:
            callbacks.remove(callback)

    def callback(self, name, *args):
        for callback in self._callbacks.get(name, []):
            callback(*args)

    # Start of Device Search Methods

    def start_search(self, interval, type_search="ssdp:all", reset=False):
        """Sends a multicast msearch to discover new devices.

        This method sends a multicast msearch message in order to discover
        new UPnP devices."""
        if reset:
            self._ssdp_server.clear_device_list()

        if not self._msearch:
            self._msearch = MSearch(self._ssdp_server, start=False)
        self._msearch.type_search = type_search
        self._msearch.start(interval)

    def stop_search(self):
        """Stop device search.
        """
        if self._msearch is not None:
            self._msearch.stop()
            self._msearch = None

    def force_discovery(self, type_search="ssdp:all"):
        """Force a multicast msearch bypassing the time interval

        This method force msearch to send discovery message, bypassing the
        initial time interval passed to start_search function. Note this method
        doesn't cause any effect if the start_search call was never called."""
        if self._msearch:
            self._msearch.type_search = type_search
            self._msearch.double_discover()

    def is_msearch_running(self):
        if self._msearch:
            return self._msearch.is_running
        return False

    # End of Device Search Methods

    # Start of SSDP Callbacks implementation

    def _new_device_event(self, st, device_info):
        # Callback assigned for new device event, processes asynchronously
        if 'LOCATION' not in device_info:
            return
        get_device_object_async(device_info['LOCATION'],
                                self._new_device_event_impl, [])

    def _new_device_event_impl(self, cargo, device_object):
        # Real implementation of the new device event
        url_info = network.parse_url(device_object.location)
        root_device = device_object.device

        # Add SOAPService to each device service
        for k, service in root_device.services.items():
            namespace = ('u', service.service_type)
            service.SOAPService = SOAPProxy(service.control_url, namespace)

        # Add SOAPService to each embedded device
        for child in root_device.devices:
            url_info = network.parse_url(child.location)
            for k, service in child.services.items():
                namespace = ('u', service.service_type)
                service.SOAPService = SOAPProxy(service.control_url, namespace)

        # This is a general on new device event callback.
        # The device listener MUST implement this method
        # and check in device_object.device.device_type
        # the device type.
        self.callback("new_device_event", device_object.device)

    def _removed_device_event(self, device_info):
        udn = device_info['USN'].split('::')[0]
        self.callback("removed_device_event", udn)

    def _on_event(self, sid, changed_vars):
        self.callback("device_event", sid, changed_vars)

    # End of SSPP Callbacks implementation
