# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from brisa import log
from brisa.control_point.control_point import ControlPoint
from brisa.control_point.control_point import get_service_control_url
from brisa.upnp.didl.DIDLLite import Element
from brisa.utils import properties


class ControlPointAV(ControlPoint):
    """This class extends ControlPoint and add basic AV functionality.
    """
    cds_ns = ('u', 'urn:schemas-upnp-org:service:ContentDirectory:1')
    avt_ns = ('u', 'urn:schemas-upnp-org:service:AVTransport:1')
    encoding = 'UTF-8'

    # Start of Properties Methods

    def _set_current_server_device(self, device):
        self._prop_current_server = device
    current_server = properties.\
                     gen_property_with_default("current_server", fset =
                                               _set_current_server_device)

    def _set_current_renderer_device(self, device):
        self._prop_current_renderer = device
    current_renderer = properties.\
                       gen_property_with_default("current_renderer", fset =
                                                 _set_current_renderer_device)

    # End of Properties Methods

    # Start of Media Server Methods

    def browse(self, object_id, browse_flag, filter, starting_index,
               requested_count, sort_criteria="dc:title"):
        """Make browses in Media Server.

        This method browser the media server with udn id. The browse_flag
        must be:
                BrowseDirectChildren
                BrowseMetadata"""
        object_id = str(object_id)
        log.debug('CP browsing id %s' % object_id)

        ss = self.current_server.SOAPService
        ss.soapaction = '%s#Browse' % self.cds_ns[1]

        browse_response = ss.Browse(ObjectID=object_id,
                                    BrowseFlag=browse_flag,
                                    Filter=filter,
                                    StartingIndex=starting_index,
                                    RequestedCount=requested_count,
                                    SortCriteria=sort_criteria)
        elt = Element.fromString(browse_response['Result'])
        browse_response['Result'] = elt.getItems()
        log.debug('browse response %s' % str(browse_response['Result']))

        return browse_response

    def search(self, container_id, search_criteria, filter, starting_index,
               requested_count, sort_criteria):
        """Search items in Media Server.

        This method search items with search_criteria key in the container_id
        of current media server."""
        ss = self.current_server.SOAPService
        ss.soapaction = '%s#Search' % self.cds_ns[1]

        search_response = ss.Search(ContainerID=container_id,
                                    SearchCriteria=search_criteria,
                                    Filter=filter,
                                    StartingIndex=starting_index,
                                    RequestedCount=requested_count,
                                    SortCriteria=sort_criteria)

        elt = Element.fromString(search_response['Result'])

        return elt.getItems()

    def get_search_capabilites(self):
        """Return the fields which the server support search.

        This method returns the fields which the server support searching."""
        ss = self.current_server.SOAPService

        ss.soapaction = '%s#GetSearchCapabilities' % self.cds_ns[1]

        try:
            return ss.GetSearchCapabilities()
        except:
            log.debug("Error while calling search capabilities function")
            return

    def get_sort_capabilities(self):
        """Returns a list of fields which the server supports sorting.

        This method returns the fields which the server support sort."""
        ss = self.current_server.SOAPService
        ss.soapaction = '%s#GetSortCapabilities' % self.cds_ns[1]

        try:
            return ss.GetSortCapabilities()
        except:
            log.debug("Error while calling sort capabilities function")
            return

    # End of Media Server Methods

    # Start of Media Renderer Methods

    def play(self, id):
        """Play item.

        Render media_url in the Renderer which udn is stored in
        self.renderer_udn."""
        if self.current_renderer:
            ss = self.current_renderer.SOAPService
            ss.soapaction = '%s#SetAVTransportURI' % self.avt_ns[1]

            item = self.browse(id, 'BrowseMetadata', '*', 0, 1, '')
            uri = item['Result'][0].uri

            ss.SetAVTransportURI(InstanceID=0,
                                 CurrentURI=uri,
                                 CurrentURIMetaData='')

            ss.soapaction = '%s#Play' % self.avt_ns[1]
            ss.Play()
        else:
            log.info("You need to choose a render")

    def stop(self):
        """Stop render.

        Stop the rendering according to the current selected renderer udn."""

        ss = self.current_renderer.SOAPService
        ss.soapaction = '%s#Stop' % self.avt_ns[1]

        ss.Stop()

    def pause(self):
        """Pause render.

        Pause the Renderer which udn is stored in self.renderer_udn."""
        if self.current_renderer is not None:
            ss = self.current_renderer.SOAPService
            ss.soapaction = '%s#Pause' % self.avt_ns[1]
            ss.Pause()
        else:
            log.info("You need to choose a render")

    def next(self):
        """Switch play next track
        """
        pass

    def previous(self):
        """Switch play to previous track
        """
        pass

    # End of Media Renderer Methods
