# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import os

from time import sleep

from brisa import log


def get_status(server_string):
    """ Gets status of a running server
    
    @param server_string: part of the name of the server that only applies to 
                          specific server. Passing "server" here will match all
                          *server* names.
                          
    @return: 'Stopped' or 'Running on PID's [list of PID's matched]'
    @rtype: string
    """
    pids = get_server_pids(server_string)
    if pids:
        status = '%s%s' % ('Running on PIDs: ', str(pids))
    else:
        status = 'Stopped'
    return status

def is_server_stopped(server_string):
    """ Returns True if the server is stopped
    
    @param server_string: part of the name of the server that only applies to 
                      specific server. Passing "server" here will match all
                      *server* names. 
    """
    return get_status(server_string) == 'Stopped'

def is_server_running(server_string):
    """ Returns True if the server is running
    
    @param server_string: part of the name of the server that only applies to 
                      specific server. Passing "server" here will match all
                      *server* names.
    """
    return not is_server_stopped(server_string)

def stop_server(server_string):
    """ Stops all servers matching with the string
    
    @param server_string: part of the name of the server that only applies to 
                      specific server. Passing "server" here will match all
                      *server* names.
    """
    if not is_server_stopped(server_string):
        pids = get_server_pids(server_string)
        for pid in pids:
            os.kill(pid, 2)

def start_server(server_string, server_app_uri):
    """ Starts a media server in a forked process
    
    @param server_string: part of the name of the server that only applies to 
                      specific server. Passing "server" here will match all
                      *server* names.
    @param server_app_uri: local address of the executable server file
    """
    if is_server_stopped(server_string):
        try:
            pid = os.fork()
            if pid == 0:
                os.execv(server_app_uri, ['', '--daemon'])
            else:
                os.wait()
        except OSError, e:
            log.error('Fail on forking process %s' % e)

def get_server_pids(server_string):
    """ Returns a list of running server PID's
    
    @param server_string: part of the name of the server that only applies to 
                          specific server. Passing "server" here will match all
                          *server* names.
    """
    pid_list = []
    append = pid_list.append
    for p in os.listdir("/proc"):
        path = "/proc/%s/cmdline" % p
        try:
            cmdline = open(path).read()
        except (IOError, OSError), e:
            continue
        if server_string in str(cmdline):
            try:
                append(int(p))
            except:
                pass
    return pid_list
