# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2005, Tim Potter <tpot@samba.org>
# Copyright 2006 John-Mark Gurney <gurney_j@resnet.uoregon.edu>
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import traceback

from brisa.upnp.service import ServiceControl
from brisa import log
from brisa.services.cds.plugin import PluginManagerSingleton
from brisa.upnp.didl.DIDLLite import Element

def compare_string(str_a, str_b):
    if str_a.title.lower().startswith("all "):
        return -1

    if str_b.title.lower().startswith("all "):
        return 1

    return cmp(str_a.title.lower(), str_b.title.lower())

class ContentDirectory(ServiceControl, dict):

    namespace = ('u', 'urn:schemas-upnp-org:service:ContentDirectory:1')

    def __init__(self):
        ServiceControl.__init__(self)
        self.plugin_manager = PluginManagerSingleton
        self.plugin_manager.listener = self
        self.updateID = 0

    def start(self):
        self.plugin_manager.load_plugins()

    def soap_GetSearchCapabilities(self, *args, **kwargs):
        """Required: Return the searching capabilities supported by
        the device."""
        log.debug('GetSearchCapabilities()')
        # While the search method is not properly implemented, the server does
        # not support search.
        return {'SearchCapabilitiesResponse': {'SearchCaps': ''}}
        #return {'SearchCapabilitiesResponse': {'SearchCaps':
        #                                       'dc:title, upnp:artist'}}

    def soap_GetSortCapabilities(self, *args, **kwargs):
        """Required: Return the CSV list of meta-data tags that can be used in
        sortCriteria."""
        log.debug('GetSortCapabilities()')
        return {'SortCapabilitiesResponse': {'SortCaps': 'dc:title'}}

    def soap_GetSystemUpdateID(self, *args, **kwargs):
        """Required: Return the current value of state variable
        SystemUpdateID."""

        log.debug('GetSystemUpdateID()')
        return {'GetSystemUpdateIDResponse': {'Id': self.updateID}}

    def soap_Browse(self, *args, **kwargs):
        args = (kwargs['ObjectID'], kwargs['BrowseFlag'], kwargs['Filter'],
                kwargs['StartingIndex'], kwargs['RequestedCount'],
                kwargs['SortCriteria'])
        l = {}
        if args[0] in self:
            l['object'] = self[args[0]]
        l['query'] = 'Browse(ObjectID=%s, BrowseFlags=%s, Filter=%s, ' \
            'StartingIndex=%s RequestedCount=%s SortCriteria=%s)' % \
            tuple(map(repr, args))
        try:
            ret = self.thereal_soap_Browse(*args)
        except defer.Deferred, x:
            ret = doRecallgen(x, self.soap_Browse, *args)

        l['response'] = ret

        return ret

    def thereal_soap_Browse(self, *args):
        """Required: Incrementally browse the native heirachy of the Content
        Directory objects exposed by the Content Directory Service."""
        (object_id, browse_flag, filter, starting_index, requested_count,
         sort_criteria) = args
        try:
            starting_index = int(starting_index)
            last_index = None
            plugin = self.plugin_manager.root_plugin
            
            if browse_flag == 'BrowseDirectChildren' and \
            requested_count != '0':
                last_index = int(requested_count) + starting_index
                
            log.debug('object_id is %s' % str(object_id))
            
            if ':' in object_id:
                namespace, object_id = object_id.split(':')
                plugin = self.plugin_manager.plugins_instances[namespace]
                
                if not plugin:
                    raise Exception("Could not get plugin associated with this \
                                    browse action")

            elements = plugin.browser(object_id, browse_flag, filter, 
                                      starting_index, requested_count, 
                                      sort_criteria)
            
            elements.sort(cmp=compare_string)
            didl = Element()
            total = 0
            
            if plugin.has_browse_filter:
                for item in elements:
                    didl.addItem(item)
                    total = total + 1
            else:
                for item in elements[starting_index : last_index]:
                    didl.addItem(item)
                    total = total + 1
                    
            didl_xml = didl.toString()
            log.debug('didl browse: %s' % didl_xml)
            log.debug(u'didl browse: %s' % unicode(didl_xml))
            soap_result = {'Result': didl_xml, 
                           'TotalMatches': len(elements),
                           'NumberReturned': total,
                           'UpdateID': self.updateID}
            soap_result = {'BrowseResponse': soap_result}
        except:
            soap_result = {'BrowseResponse': {'Result': '',
                                              'TotalMatches': 0,
                                              'NumberReturned': 0,
                                              'UpdateID': self.updateID}}
            traceback.print_exc()

        return soap_result

    # Optional actions
    #TODO: Fixes the soap search function
    #each plugin should implement its own search function
    #THIS CODE IS BROKEN
    def soap_Search(self, *args, **kwargs):
        """Search for objects that match some search criteria."""

        (ContainerID, SearchCriteria, Filter, StartingIndex,
         RequestedCount, SortCriteria) = args

        didl = Element()

        ContainerID = int(ContainerID)

        total = 0

        wantedTitle = "None"
        objects = Facade.search_object_by_contain_in_title(wantedTitle)

        if "dc:title" in SearchCriteria:
            looked = SearchCriteria.split("dc:title")[1]
            if looked.find("=") is not -1:
                wantedTitle = looked.split("=")[1]
                wantedTitle = wantedTitle.strip('")( ')

            if "contains" in SearchCriteria:
                wantedTitle = looked.split("contains")[1]
                wantedTitle = wantedTitle.strip('")( ')

            objects = Facade.search_object_by_contain_in_title(wantedTitle)
            if objects.count() is not 0:
                for obj in objects[int(StartingIndex): int(StartingIndex)+ \
                                   int(RequestedCount)]:
                    # Container always have a id number less than its Items
                    if obj.id <= ContainerID:
                        pass
                    else:
                        if isinstance(obj, Persistence.Container):
                            obj.setParentId(obj.container.id)
                        else:
                            obj.setParentId(obj.containers[0].id)
                        didl.addItem(obj)
                        total = total + 1
        if "upnp:artist" in SearchCriteria:
            looked = SearchCriteria.split("upnp:artist")[1]
            if looked.find("=") is not -1:
                wantedTitle = looked.split("=")[1]
                wantedTitle = wantedTitle.strip('")( ')

            if "contains" in SearchCriteria:
                wantedTitle = looked.split("contains")[1]
                wantedTitle = wantedTitle.strip('")( ')

            objects = Facade.select_music_artist_by_title(wantedTitle)
            if objects.count() is not 0:
                for obj in objects[int(StartingIndex): int(StartingIndex)+ \
                                   int(RequestedCount)]:
                    # Container always have a id number less than its Items
                    if obj.id <= ContainerID:
                        pass
                    else:
                        if len(obj.items) is not 0:
                            for i in obj.items[int(StartingIndex):\
                                               int(StartingIndex)+ \
                                               int(RequestedCount)]:
                                i.setParentId(str(obj.id))
                                didl.addItem(i)
                                total = total + 1

        log.info('Search(ContainerID=%s, SearchCriteria=%s, Filter=%s, ' \
            'StartingIndex=%s, RequestedCount=%s, SortCriteria=%s)',
            ContainerID, SearchCriteria, Filter, StartingIndex,
            RequestedCount, SortCriteria)

        r = {'Result': didl.toString(), 'TotalMatches': total,
                'NumberReturned': didl.numItems(), }

        r['UpdateID'] = 0

        result = {'SearchResponse': r}

        return result

    def on_start_plugin(self, plugin_name):
        log.info('Media Server received START plugin event :: %s',
                     plugin_name)

    def on_end_plugin(self, plugin_name):
        log.info('Media Server received END plugin event :: %s',
                     plugin_name)
