# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from brisa import log
from brisa import config
from brisa.main import ThreadManager, ThreadObject
from brisa.upnp.didl.DIDLLite import Object, Container

log = log.getLogger("PluginManager")

class Plugin(object):
    '''The base class to all plugins.'''

    name = 'plugin_stub'
    usage = False
    watch = False
    has_browse_filter = False

    def __init__(self, *args, **kwargs):
        '''
        Initialize the base plugin.

        Arguments:
        args - The configuration parameters to the loadded plugin.
        '''
        self.plugin_manager = PluginManagerSingleton

    def execute(self):
        '''Load the plugin media information into the database.'''

        self.load()
        self.plugin_manager.notify_plugin_loaded(self.name)

    def load(self):
        '''Force subclasses implement the load method'''

        raise NotImplementedError("%s: load method not implemented" %
                                  self.name)

    def unload(self):
        '''Load the plugin media information into the database.'''

        raise NotImplementedError("%s: unload method not implemented" %
                                  self.name)

    def browser(self, str_object_id, browse_flag, filter, starting_index,
                requested_count, sort_criteria):
        raise NotImplementedError("browser not implemented for plugin %s" %
                                  self.name)

    def search(self, str_object_id, browse_flag, filter, starting_index,
                requested_count, sort_criteria):
        raise NotImplementedError("search not implemented for plugin %s" %
                                  self.name)

class MemObject(object):

    def __init__(self, id, title, parent_container_id='-1'):
        self.id = id
        self.title = title
        self.parent_container_id = parent_container_id

    def __repr__(self):
        return self.title

class MemDIDLObject(Object):

    def __init__(self, *args, **kargs):
        self.didlobject = MemObject(*args, **kargs)

class MemContainer(MemObject):

    def __init__(self, id, title, parent_container_id='-1'):
        MemObject.__init__(self, id, title, parent_container_id)

    def _get_childCount(self):
        return len(self.containers)

    childCount = property(_get_childCount)

class MemDIDLContainer(Container):

    def __init__(self, *args, **kargs):
        Container.__init__(self, *args, **kargs)
        self.didlobject = MemContainer(*args, **kargs)

    def add_child(self, container):
        container.parent_container_id = self.didlobject.id
        self.containers.append(container)

class _RootPlugin(Plugin):
    name = "rootplugin"
    containers = {}

    def __init__(self):
        pass

    def load(self):
        self.add_container("brisaServerRoot", "0")
        self.add_container("Music", "1", "0")
        self.add_container("Pictures", "3", "0")
        self.add_container("Playlists", "12", "0")
        self.add_container("Video", "2", "0")
        self.add_container("Audio Broadcast", "4", "1")
        self.add_container("Video Broadcast", "5", "2")
        self.add_container("Remote Pictures", "6", "3")

    def unload(self):
        pass

    def browser(self, id, browse_flag, filter, starting_index, 
                requested_count, sort_criteria):
        log.debug('Browser: id= %s, browse_flag= %s, filter= %s,\
        starting_index= %s, requested_count= %s, sort_criteria= %s',
        id, browse_flag, filter, starting_index, requested_count,
        sort_criteria)
        if browse_flag == "BrowseDirectChildren":
            return self.containers[id].containers
        else:
            return [self.containers[id]]

    def search(self, id, browse_flag, filter, starting_index, requested_count,
                sort_criteria):
        pass

    def add_container(self, title, object_id, parent_container_id="-1", plugin=None):
        object_id = str(object_id)
        parent_container_id = str(parent_container_id)
        if plugin != None:
            if object_id == None:
                raise Exception("Error adding root plugin container: you must \
define the object_id")
            if not hasattr(plugin, "browser") or not hasattr(plugin,"name"):
                raise NotImplementedError("The plugin must implement the \
'browse' method and assign an attribute 'name'.")
            object_id = "%s:%s" % (plugin.name, object_id)

        new_container = MemDIDLContainer(id=object_id, 
                                         title=title,
                                         parent_container_id=parent_container_id)
        self.containers[object_id] = new_container

        if parent_container_id != "-1":
            self.containers[parent_container_id].add_child(new_container)
        return new_container

RootPluginSingleton = _RootPlugin()


class PluginLoader(ThreadObject):

    plugin = None

    def __init__(self, plugin):
        ThreadObject.__init__(self)
        self.plugin = plugin

    def run(self):
        try:
            self.plugin.execute()
        except:
            log.error("ERROR loading %s plugin\n", self.plugin.name)
            import traceback
            traceback.print_exc()
            print ""
            
    def prepare_to_stop(self):
        # Probably done somewhere else.
        self.plugin.unload()

class _PluginManager(object):
    '''Manages the loading of plugins.'''
    # Plugins instances
    plugins_instances = {}

    # Plugins classes
    plugins_classes = {}

    def __init__(self):
        from brisa.services.cds import plugins
        self.root_plugin = RootPluginSingleton
        self.root_plugin.load()
        self.plugins_classes[self.root_plugin.name] = self.root_plugin.__class__
        self._recognize_plugins()
        self.listener = None

    def load_plugins(self):
        self._instanciate_plugins()
        self.exec_plugins()
        log.debug('Available plugins: %s', str(self.plugins_classes.keys()))

    def _recognize_plugins(self):
        classes = Plugin.__subclasses__()

        plugins_class_ref = self.plugins_classes.values()
        for _class in classes:
            if _class not in plugins_class_ref:
                self.plugins_classes[_class.name] = _class

    def _instanciate_plugins(self):
        '''Load all plugin classes into the plugin manager.'''
        plugins = self.plugins_classes.values()
        for plugin in plugins:
            if plugin.usage:
                try:
                    self.plugins_instances[plugin.name] = plugin()
                except:
                    log.error("Error while instantiating %s:", plugin.name)
                    import traceback
                    traceback.print_exc()
                    print ""

    def load_plugin(self, plugin):
        plugins_loader = PluginLoader(plugin)
        plugins_loader.setName("BRisa Plugin Loader (%s)" % plugin.name)
        plugins_loader.start()

    def exec_plugins(self):
        '''Execute the selected plugins'''
        try:
            plugins = self.plugins_instances.values()
            for plugin in plugins:
                log.info("START loading plugin %s", plugin.name)
                self.load_plugin(plugin)
                if self.listener is not None:
                    self.listener.on_start_plugin(plugin.name)
        except Exception, e:
            log.show_exception("Error while loading plugin %s" % plugin.name,
                                   e, 1)

    def notify_plugin_loaded(self, plugin_name):
        log.info("END loading plugin %s", plugin_name)
        if self.listener is not None:
            self.listener.on_end_plugin(plugin_name)

PluginManagerSingleton = _PluginManager()
