# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from Persistence import *
from Persistence import create_db as persistence_create_db


# Database Manipulation methods

# General Methods

def get_next_object_id():
    """ Returns the next object id according to actual database status.
    """
    queue_manager.put_and_join('select count(*) from db_object')
    return queue_manager.task_result[0][0] + 1

def remove_all_invalid_items():
    """ Removes all invalid items (which have the valid attribute set to 0) out
    from the database.
    """
    queue_manager.put_and_join('DELETE FROM db_item WHERE valid=0')

def sync_database():
    """ Initial step for synchronizing system with the database. 
    
    In this step all objects have their 'valid' attributes set to '0'. 
    
    In a later stage, the responsible plugin for each file will see if the data
    on the database is consistent, that is, the file still exists on the system
    and its database information are consistent. If consistent, valid is set 
    back to '1' so that consults on the database for this runtime context are
    correct.
    """
    make_all_objects_invalid()

def create_db():
    """ Creates all tables.
    """
    persistence_create_db()



# Objects

def get_object_id_by_title(title):
    """ Returns an object id given its title
    """
    return select_object_by_title(title).id

def select_object_by_id(id):
    """ Selects an object given its id on the table
    
    @param id: object id on table
    
    @return: a DBObject associated with the database entry
    @rtype: DBObject or None
    """
    debug('select object by id %d' % id)
    try:
        return get_by_id(id, DBObject)
    except Exception, e:
        debug('select object by id ERROR %s' % str(e))
        return None

def select_object_by_title(title):
    debug('select object by title %s' % title)
    
    try:
        return get_by_title(title, force_class=DBObject)
    except Exception, e:
        debug('select object by title ERROR %s | %s' % (title, str(e)))
        
def make_all_objects_invalid():
    """ Sets the 'valid' attribute of all DBItem to 0.
    
    See brisa.services.cds.plugins.filesystem.persistence.Facade.sync_database
    documentation for further explanation.
    """
    queue_manager.put_and_join('UPDATE db_item SET valid = 0 where valid = 1')
    

def remove_object_by_id(id):
    """ Removes an object from the database given its id
    
    @return: boolean True if removed successfully
    """
    queue_manager.put_and_join("DELETE FROM db_object WHERE id='%d'" % id)
    
    # Maybe we don't need this check.
    queue_manager.put_and_join("SELECT id FROM db_object WHERE id='%d'" % id)

    k = queue_manager.task_result
    
    if not k:
        # List is empty, success
        debug('remove object by id %d (succeeded)' %d)
        return True
    
    debug('remove object by id %d (failed)' % id)
    return False


def search_object_by_contain_in_title(title):
    debug('TODO search object by contain in title %s' % title)


def search_object_by_exactly_title(title):
    debug('TODO search object by exactly title %s' % title)


def search_object_by_description(description):
    debug('TODO search object by description' % description)
    

# Containers

def add_container(title, namespace, parent_container_id=0):
    """ Adds a new Container to the database.
    
    @param title: title
    @param namespace: owner plugin name
    @param parent_container_id:
    
    @type               title: string
    @type           namespace: string
    @type parent_container_id: integer
    
    @return DBContainer object representing it or None (in case of failure)
    """
    
    debug('add container %s %s %s' % (title, namespace, parent_container_id))
    
    # Add the container, then the DBObject
    
    # Calculate id: since the id of this DBContainer is the same as his 
    # associate object, we get the next object id :)
    id = get_next_object_id()
    
    debug('add container %s next id is %d' % (title, id))
    
    container = None
    
    if parent_container_id > 0:
        # If it has a parent_container, get it
        parent_container = select_container_by_id(parent_container_id)
        
        if parent_container:
            debug('add container has parent and selected it correctly')
            # Manage to select, then pass it
            container = DBContainer(id=id,
                                    parent_container_id=parent_container_id,
                                    title=title,
                                    namespace=namespace,
                                    parent_container=parent_container)
        else:
            debug('add container has parent and could not select it, creating')
            # Didn't select, parent doesn't exist! create it
            parent_container = DBContainer(id=parent_container_id, 
                                           namespace=namespace)
            parent_container.save()
            container = DBContainer(id=id,
                                    parent_container_id=parent_container_id,
                                    title=title,
                                    namespace=namespace,
                                    parent_container=parent_container)
    else:
        # Doesn't have parent_container, so its a root
        debug('add container root clause')
        container = DBContainer(id=id,
                                parent_container_id=0,
                                title=title, 
                                namespace=namespace)
        
    if not container:
        # If we didn't get a container yet, bug happened!
        debug('add container BUG (could not create container')
        return None

    # Save this guy
    container.save()
    
    return container


def select_container_by_title(title, skip_children=True):
    """Select container by title.

    Select a Container with title passed."""
    return get_by_title(title, skip_children)
    
    
def remove_container_by_id(id):
    """ Removes a container from database given its id. This means that both
    db_container and db_object associated will be deleted
    
    @return: boolean True if removed successfully
    """
    # Remove from db_object table
    remove_object_by_id(id)
    
    # Remove from db_container table
    queue_manager.put_and_join("DELETE FROM db_container WHERE id='%d'" % id)
    
    # Check if removed correctly
    queue_manager.put_and_join("SELECT count(*) FROM db_container WHERE id='%d'"% id)
    count = queue_manager.task_result[0][0]
    
    if not count:
        # Empty list means deleted successfully
        debug('remove container by id %d (succeeded)' % id)
        return True
    
    debug('remove object by id %d (failed)' % id)
    return False


def select_containers_from_container(container):
    """Select container from container(DB).

    Select a Container by SQLmethod selectBy.
    Returns the Container selected """
    debug('TODO select containers from container %s' % str(container))


def has_object_by_title(title):
    queue_manager.put_and_join('SELECT id from db_object WHERE title="%s"' % title)
    
    items = queue_manager.task_result
    
    if len(items) > 0:
        debug('has object by title %s: %s' % (title, True))
        return True
    
    debug('has objecth by title %s: %s' % (title, False))
    return False


# Items

def remove_container_items_by_local_path(local_path):
    debug('TODO remove container items by local path %s' % local_path)


def remove_container_items_by_name(name):
    debug('TODO remove container items by name %s' % name)


def remove_all_items_from_container(container):
    """ Removes all items from a container
    """
    debug('TODO remove all items from container %s' % str(container))


def clean_container(container):
    """ Removes all items and all containers from the given container
    """
    debug('TODO clean container %s' % str(container))
        
        
def select_item_by_local_path(local_path):
    """ Select an item given its local path. Returns a list of DBItem instances
    with data from this select

    @param local_path: item's path
    @type local_path: string
    
    @return: items selected
    @rtype: list or None
    """
    
    queue_manager.put_and_join('SELECT id FROM db_item WHERE local_path="%s"' % local_path)
    
    try:
        id = queue_manager.task_result[0][0]
        return get_by_id(id)
    except:
        return None 


def select_all_items():
    """ Select all Items of UPnP strucuture."""
#    return DBItem.select()
    debug('select all items')


def select_item_by_id(itemId):
    """ Selects an item given its id
    """
    # This is just returning the first match, since ID's are unique
    try:
        debug('select item by id %s' % str(itemId))
        return get_by_id(int(itemId), DBItem)
    except Exception, e:
        debug('select item by id %s error %s' % (str(itemId), str(e)))
        return None


def has_item_by_local_path(local_path, sizebytes=0):
    """Check if local item exist.

    This method analyzes if exist a Item in the specified path.
    Return: count - The number of items found.
    """
    items = []
    
    debug('local item exist %s %d' % (local_path, sizebytes))
        
    if sizebytes:
        queue_manager.put_and_join(u'SELECT local_path FROM db_item WHERE local_path="%s" and sizebytes="%s"' % (local_path, sizebytes))
        items = queue_manager.task_result
        debug('local item exist %s %d: %s' % (local_path, sizebytes, len(items)>0))
    else:
        queue_manager.put_and_join(u'SELECT local_path FROM db_item WHERE local_path="%s"'\
                                 % local_path)
        items = queue_manager.task_result
        
        debug('local item exist %s: %s' % (local_path, len(items)>0))

    return len(items) > 0


# Genre

def remove_genre_by_id(id):
    """ Removes a genre from the database given its id.
    """
    # TODO must remove object and also container
    debug('FIXME remove genre by id %d' % id)
    return remove_object_by_id(id)


def add_music_genre(title, namespace, parent_container_id):
    """ Add a music genre to the database
    
    @param title: container title
    @param namespace: namespace
    @param parent_container_id: id of parent container
    """
    debug('add music genre %s %s %s' % (title, namespace, str(parent_container_id)))

    music_genre = None
    
    if not has_object_by_title(title):
        debug('add music genre does not have container')
        # If not created yet, create it
        id = get_next_object_id()
        music_genre = DBMusicGenre(id=id,
                                   name=title, 
                                   parent_container_id=parent_container_id,
                                   title=title,
                                   namespace=namespace)
        music_genre.save()
    return music_genre


def remove_music_genre_by_id(id):
    """ Removes a music genre from the database
    """
    # TODO remove also the container from db_container, this just removes
    # the object from db_object
    # FIXME should this be returning an object? check calls to this function
    debug('CHECKME remove music genre by id %d' % id)
    return remove_object_by_id(id)


def add_movie_genre(title, container=None):
    """ Adds a movie genre to the database
    """
    # TODO check if it already has the movie genre before adding
    # TODO check params

    debug('add movie genre %s %s' % (title, str(container)))
    
    queue_manager.put_and_join("INSERT INTO db_movie_genre(type) values ('%s')" % (title))
        
    return DBContainer(title=title, container=container)


def remove_movie_genre_by_id(id):
    """ Removes a movie genre from the database
    """
    # CHECKME should this be returning an object? check calls to this function
    debug('CHECKME remove movie genre by id %d' % id)
    return remove_object_by_id(id)


# Music

def add_music_artist(title, namespace, parent_container_id):
    """ Adds a music artist to the database
    """
    debug('add music artist %s %s %s' % (title, namespace, str(parent_container_id)))
    
    if not has_object_by_title(title):
        music_artist = DBMusicArtist(id=get_next_object_id(),
                                     title=title,
                                     name=title,
                                     namespace=namespace,
                                     parent_container_id=parent_container_id)
        music_artist.save()
        return music_artist
    
    return None


def remove_music_artist_by_id(id):
    """ Removes a music artist from the database given its id
    """
    # CHECKME should this be returning an object? check calls to this function
    return remove_object_by_id(id)


def add_music_album(title, namespace, parent_container_id):
    """ Adds a music album to the database
    """
    debug('add music album %s %s %s' % (title, namespace, str(parent_container_id)))

    if not has_object_by_title(title):
        music_album = DBMusicAlbum(id=get_next_object_id(),
                                   title=title,
                                   name=title,
                                   namespace=namespace,
                                   parent_container_id=parent_container_id)
        music_album.save()
        return music_album
    
    return None


def remove_music_album_by_id(id):
    """ Removes a music album from the database given its id
    """
    # TODO also remove the container from db_container
    # CHECKME should this be returning an object? check calls to this function
    debug('CHECKME remove music album by id %d' % id)
    return remove_object_by_id(id)


def add_music_item(**music_template):
    """ DEPRECATED: don't use this directly instead you know exactly what you
    are doing. This shouldn't be used since when you create higher level classes
    they automatically create their inherited classes.
    
    Adds a music item to the database given a dict (music_template) with
    its information.
    """
    debug('add music item with template: %s' % str(music_template))
    music_item = DBMusicItem(**music_template)
    music_item.save()
    return music_item
    

def add_music_track(**music_data):
    """ Add a music track to the database given a dict with necessary info. Adds
    also genre, artist and album if they're not on the database yet.
    """
    
    debug('add music track music_template: %s' % str(music_data))
    
    music_genre = None
    music_artist = None
    music_album = None
    music_all = None
    
    music_genre_parent = None
    music_audio_artist_parent = None
    music_album_parent = None
    
    music_genre = get_by_title(music_data['genre'], True)
    if not music_genre:
        music_genre_parent = select_container_by_title('Audio Genres')
        
        if music_genre_parent:
            music_genre = add_music_genre(music_data["genre"],
                                         music_data['namespace'],
                                         music_genre_parent.id)
        else:
            debug('add music track critical 1 | dump: %s' % str(music_data))
        
    music_artist = get_by_title(music_data['artist'], True)
    if not music_artist:
        music_audio_artist_parent = select_container_by_title('Audio Artists')
        
        if music_audio_artist_parent:
            music_artist = add_music_artist(music_data['artist'], 
                                           music_data['namespace'],
                                           music_audio_artist_parent.id)
        else:
            debug('add music track critical 2 | dump: %s' % str(music_data))
        
    music_album = get_by_title(music_data['album'], True)
    if not music_album:
        music_album_parent = select_container_by_title('Audio Albums')
        
        if music_album_parent:
            music_album = add_music_album(music_data['album'], 
                                         music_data['namespace'],
                                         music_album_parent.id)
        else:
            debug('add music track critical 3 | dump: %s' % str(music_data))

    music_all = select_container_by_title('All Music')

    if not music_all:
        # Perhaps we should create it
        debug('add music track critical 4 | dump %s' % str(music_data))

    # Sets a few important variables
    music_data['id'] = get_next_object_id()
    music_data['parent_container_id'] = music_album.id    
    music_data['artist'] = music_artist.name
    music_data['album'] = music_album.name
    music_data['name'] = music_data['title']
    
    # Add the music track and save it
    music_track = DBMusicTrack(**music_data)
    music_track.save()
    
    # Add to respective containers
    music_genre.add_item(music_track)
    music_artist.add_item(music_track)
    music_album.add_item(music_track)
    music_all.add_item(music_track)
    
    # Save containers (this saves the relations between the containers and this
    # item)
    music_genre.save()
    music_artist.save()
    music_album.save()
    music_all.save()
    
    return music_track


def remove_music_track_by_id(id):
    """ Remove a music track from the database given its id
    """
    # CHECKME
    # TODO remove container if it will be empty? 
    debug('CHECKME remove music track by id %d' % id)
    return remove_object_by_id(id)


def add_music_video_clip(*args, **kwargs):
    """ Add a music video to the database
    """
    # FIXME parameters list
    debug('add music video clip args: %s' % str(args))
    debug('add music video clip kwargs: %s' % str(kwargs))
    music_video_clip = DBMusicVideoClip(*args, **kwargs)
    return music_video_clip 


def add_movie(**movie_data):
    """ Add a movie to the database given
    """
    if not has_object_by_title(movie_data['genre']):
        # todo
        movie_genre = add_movie_genre()
    else:
        movie_genre = select_genre_by_title(movie_data['genre'])

    del movie_data["genre"]
    movie_data["movie_genre"] = movie_genre
    
    movie = DBMovie(**movie_data)
    movie.save()
    
    movie_genre.add_item(movie)
    movie_genre.save()
    return movie


def remove_movie_by_id(id):
    """ Removes a movie from the database given its id
    """
    # CHECKME is there a container to be removed if it will get empty after
    #         removing this movie?
    #
    # CHECKME should this be returning an object? check calls to this function
    # TODO add debug message
    return remove_object_by_id(id)


def add_video_album(title, parent_container_id):
    """ Adds a video album to the database
    """
    debug('add video album %s %s' % (title, str(parent_container_id)))
    video_album = DBVideoAlbum(id=get_next_object_id(),
                                                         title=title,
                                                         name=title, 
                                                         parent_container_id=parent_container_id)
    video_album.save()
    return video_album


def add_video_item(**video_data):
    """ Add a video item to the database given a data dict, which must comply
    to this video data dict definition
    """
    debug('add video item video_data: %s' % str(video_data))
    
    video_container = None
    video_container_all = None
    
    if not has_object_by_title(video_data['root']):
        parent_container = select_container_by_title('Video Folders')
        video_container = add_video_album(video_data['root'], 
                                                                        parent_container.id)
    else:
        video_container = select_container_by_title(video_data['root'])
        
    video_container_all = select_container_by_title('All Video')

    if not video_container or not video_container_all:
        debug('add video item bug detected!')
        return None
    
    video_data['id'] = get_next_object_id()
    video_data['name'] = video_data['title']
    video_data['parent_container_id'] = video_container.id
    
    video = DBVideoItem(**video_data)
    video.save()
    
    video_container.add_item(video)
    video_container_all.add_item(video)
    
    video_container.save()
    video_container_all.save()
    return video


def remove_music_video_clip_by_id(id):
    """ Remove a music video clip from the database given its id
    """
    # CHECKME check if we don't need to remove the parent container
    # CHECKME should this be returning an object? check calls to this function
    return remove_object_by_id(id)


def remove_text_by_id(id):
    """ Remove a text item from the database 
    """
    # CHECKME check if we don't need to remove the parent container
    # CHECKME should this be returning an object? check calls to this function    
    return remove_object_by_id(id)


def add_photo_album(title, namespace, parent_container_id=0):
    """ Add a photo folder/album to the database
    """
    debug('add photo folder %s %s %s' % (title, namespace, str(parent_container_id)))
    if not has_object_by_title(title):
        photo_album = DBPhotoAlbum(id=get_next_object_id(),
                                                               namespace=namespace,
                                                               name=title,
                                                               parent_container_id=parent_container_id)
        photo_album.save()
        return photo_album
    else:
        debug('add photo folder did not add because db already has that genre')
        return None


def add_photo(**photo_data):
    """ Add a photo item to the database
    """
    debug('add_photo: %s' % str(photo_data))
    
    photo_album = None
    photo_container_all = None
    
    if not has_object_by_title(photo_data['root']):
        parent_container = select_container_by_title("Local Images")
        photo_album = add_photo_album(photo_data['root'],
                                                                        photo_data['namespace'],
                                                                        parent_container.id)
    else:
        photo_album = select_container_by_title(photo_data["root"])

    photo_container_all = select_container_by_title("All Pictures")

    photo_data['id'] = get_next_object_id()
    photo_data['parent_container_id'] = photo_album.id
    photo_data['album'] = photo_album.name
    photo = DBPhoto(**photo_data)
    photo.save()
    
    photo_album.add_item(photo)
    photo_container_all.add_item(photo)
    
    photo_album.save()
    photo_container_all.save()
    return photo


def remove_photo_by_id(id):
    """ Remove a photo item from the database given its id
    """
    # TODO remove parent container if it gets empty?
    # CHECKME should this be returning an object? check calls to this function
    # TODO add debug message
    return remove_object_by_id(id)


def remove_item_by_local_path(local_path):
    """ Remove an item from the database given its path
    """
    # CHECKME check this, probably not working
    # TODO add debug message
#    item = DBItem.selectBy(local_path=local_path)
#    if item.count() > 0:
#        item[0].destroySelf()
    debug('TODO remove item by local path %s' % local_path)



# Selection methods

def select_audio_list():
    """ Selects audio list
    """
    # TODO returning two types is ugly...
    # TODO add debug message
    debug('select audio list')
    try:
        queue_manager.put_and_join("SELECT title FROM db_object WHERE type='DBItem'")    
        return queue_manager.task_result
    except:
        return False


def select_genre_by_title(title):
    """ Selects a genre by its title
    """
    debug('select genre by title %s' % title)
    genre = get_by_title(title)
    return genre


def select_all_genres():
    """Select all genre."""
#    return DBGenre.select()
    debug('select all genres')


def select_all_movie_genre():
    """Select all movie genre.

    Select all Movie Genre. This function calls the atribute select of
    MovieGenre to do it."""
#    return DBMovieGenre.select()
    debug('select all movie genre')


def select_all_music_artists():
    """Select all Music Artist.

    Select all Music Artists. This function calls the atribute select of
    Music Artist to do it. """
#    return DBMusicArtist.select()
    debug('select all music artists')


def select_all_music_album():
    """Select all Music Album.

    Select all Music Album. This function calls the  atribute select of
    Music Album to do it. """
#    return DBMusicAlbum.select()
    debug('select all music album')


