# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
#
# Copyright (C) 2006 Fluendo, S.A. (www.fluendo.com).
# Copyright 2006, Frank Scholz <coherence@beebits.net>
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from brisa.utils.network import parse_http_response
from brisa.utils.network_senders import UDPTransport
from brisa.utils.network_listeners import UDPListener
from brisa.utils.python.looping_call import LoopingCall

from brisa import log

DEFAULT_SEARCH_TIME = 600.0

class MSearch(object):

    def __init__(self, ssdp, start=True, interval=DEFAULT_SEARCH_TIME,
                 ssdp_addr='239.255.255.250', ssdp_port=1900):
        # Sharing sockets between UDPTransport and UDPListener is CRITICAL
        # here. It must be done so that the answer to M-Search comes back to
        # this listener.
        self.transport = UDPTransport()
        self.listenUDP = UDPListener(ssdp_addr, ssdp_port, 
                                     data_callback=self.datagram_received,
                                     shared_socket=self.transport.socket)
        self.ssdp = ssdp
        self.ssdp_addr = ssdp_addr
        self.ssdp_port = ssdp_port
        self.type_search = "ssdp:all"
        self.loopcall = LoopingCall(self.double_discover)
        if start:
            self.start(interval)

    def _running(self):
        return self.loopcall.running

    is_running = property(fget=_running)

    def start(self, interval=DEFAULT_SEARCH_TIME):
        log.debug('MSearch started')
        self.listenUDP.start()
        if not self.loopcall.running:
            log.debug('MSearch loopcall started')
            self.loopcall.start(interval, now=True)

    def stop(self):
        self.listenUDP.stop()
        if self.loopcall.running:
            self.loopcall.stop()

    def datagram_received(self, data, (host, port)):
#        log.debug('MSearch: datagram received from (%s, %s): %s\n' % (str(host), str(port), data))
        cmd, headers = parse_http_response(data)
        if cmd[0] == 'HTTP/1.1' and cmd[1] == '200':
            if self.ssdp != None:
                if not self.ssdp.is_known_device(headers['usn']):
                    self.ssdp.register(headers['usn'], headers['st'],
                                              headers['location'],
                                              headers['server'],
                                              headers['cache-control'])

    def double_discover(self):
        self.discover(self.type_search)
        self.discover(self.type_search)

    def discover(self, type = "ssdp:all"):
        req = ['M-SEARCH * HTTP/1.1',
                'HOST: %s:%d' % (self.ssdp_addr, self.ssdp_port),
                'MAN: "ssdp:discover"',
                'MX: 5',
                'ST: ' + type, '', '']
        req = '\r\n'.join(req)
        self.transport.send_data(req, (self.ssdp_addr, self.ssdp_port))
