# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.

# Copyright 2006, Elisa team <contact@fluendo.com>
# Copyright 2007-2008, Brisa Team <brisa-develop@garage.maemo.org>

from xml.etree import ElementTree
from brisa import log
from brisa.main import run_async_call
from brisa.utils import properties
from brisa.utils.network import url_fetch, parse_url, http_call
from brisa.upnp.upnp_publisher import UPnPPublisher
from brisa.upnp.upnp_defaults import UPnPDefaults
from brisa.services.web_server import CustomResource, StaticFile


class ServiceControl(UPnPPublisher, object):
    pass


class Service(object):

    def __init__(self, location, control_url,
                 event_sub_url, presentation_url, scpd_url):
        self._actions = {}
        self._variables = {}
        self._var_subscribers = {}
        self._prop_friendly_name = ""
        self.description_xml_filepath = ""
        self.service_type = ""
        self.event_sid = ""

        self.timeout = 0
        self.location = location
        self.control = None

        parsed = parse_url(self.location)
        self.url_base = "%s://%s" % (parsed[0], parsed[1])

        self.control_url = self._format_url(control_url)
        self.event_sub_url = self._format_url(event_sub_url)
        self.presentation_url = self._format_url(presentation_url)
        self.scpd_url = self._format_url(scpd_url)

        self.friendly_name = self.control_url.split("/")[1]

    def actions(self):
        return self._actions

    def variables(self):
        return self._variables

    def var_subscribers(self):
        return self._var_subscribers

    def build(self):
        self.async = False
        self.callback = None
        self.parse_service_description(self.scpd_url)

    def build_async(self, callback):
        self.async = True
        self.callback = callback
        self.parse_service_description(self.scpd_url)

    def _format_url(self, url):
        if url is not None:
            if not url.startswith('/') and not url.startswith('http'):
                # relative URL without leading /
                url = "/%s" % url
            if not url.startswith('http'):
                # relative URL
                url = "%s%s" % (self.url_base, url)
        return url

    def _set_service_id(self, service_id):
        prefix = UPnPDefaults.SERVICE_ID_PREFIX
        if service_id == 'X_MS_MediaReceiverRegistrar':
            prefix = UPnPDefaults.SERVICE_ID_MS_PREFIX
        self._prop_service_id = '%s%s' % (prefix, service_id)

    service_id = properties.gen_property_with_default("service_id",
                                                       fset=_set_service_id)

    def _set_subscription_id(self, subscription_id):
        self._prop_subscription_id = subscription_id
        if subscription_id:
            subscribe(self)

    subscription_id = properties.gen_property_with_default(
                                                  "subscription_id",
                                                  fset=_set_subscription_id)

    def get_state_variable(self, name):
        return self._variables.get(name)

    def subscribe(self):
        event.subscribe(self)
        global subscribers
        subscribers[self.get_sid()] = self

    def unsubscribe(self):
        pass

    def subscribe_for_variable(self, var_name, callback):
        variable = self.get_state_variable(var_name)
        if variable:
            variable.subscribe(callback)

    def renew_subscription(self):
        pass

    def fetch_description(self, path):
        if not path:
            return None

        xmldata = url_fetch(path)

        if not xmldata:
            return None

        try:
            etree = ElementTree.ElementTree(file=xmldata)
            tree = etree.getroot()
        except:
            tree = ElementTree.XML("<xml></xml>")

        return tree

    def fetch_description_async(self, path):
        if not path:
            self.fetch_description_async_done("<xml></xml>")

        if brisa.__skip_service_xml__:
            self.fetch_description_async_done("<xml></xml>")
            return

        run_async_call(url_fetch, self.fetch_description_async_done,
                       self.fetch_description_async_error, delay=0, url=path)

    def fetch_description_async_error(self, error):
        log.debug("Error fetching service description")
        self.fetch_description_async_done("<xml></xml>")
        return True

    def fetch_description_async_done(self, fd):
        try:
            tree = ElementTree.XML(file=fd).getroot()
        except:
            tree = None

        self.parse_actions(tree)
        self.callback(self)

    def parse_actions(self, tree):
        ns = "urn:schemas-upnp-org:service-1-0"

        for action_node in tree.findall('.//{%s}action' % ns):
            name = action_node.findtext('{%s}name' % ns)
            arguments = []
            for argument in action_node.findall('.//{%s}argument' % ns):
                arg_name = argument.findtext('{%s}name' % ns)
                arg_direction = argument.findtext('{%s}direction' % ns)
                arg_state_var = \
                    argument.findtext('{%s}relatedStateVariable' % ns)
                arguments.append(Argument(arg_name, arg_direction,
                                          arg_state_var))

            self._actions[name] = Action(self, name, arguments)

        for var_node in tree.findall('.//{%s}stateVariable' % ns):
            events = ["no", "yes"]
            send_events = events.index(var_node.attrib["sendEvents"])
            name = var_node.findtext('{%s}name' % ns)
            data_type = var_node.findtext('{%s}dataType' % ns)
            values = []
            for allowed in var_node.findall('.//{%s}allowedValue' % ns):
                values.append(allowed.text)
            self._variables[name] = StateVariable(self, name, send_events,
                                                  data_type, values)

    def parse_service_description(self, path):
        if self.async:
            self.fetch_description_async(path)
        else:
            xml = self.fetch_description(path)
            if xml:
                self.parse_actions(xml)

    def start_service(self):
        if self.control is None:
            raise Exception('missing control functions. Set the control \
attribute.')

        service_resource = CustomResource()
        service_resource.add_static_file('scpd.xml',
            StaticFile(self.description_xml_filepath))
        service_resource.add_resource('control', self.control)
        return service_resource

    def event_subscribe(self, event_host, callback, cargo):
        if self.event_sid:
            # already registered
            return
        SubscribeRequest(self, event_host, callback, cargo)

    def event_unsubscribe(self, event_host, callback, cargo):
        if not self.event_sid:
            # not registered
            return
        UnsubscribeRequest(self, event_host, callback, cargo)


class Argument:

    def __init__(self, arg_name, arg_direction, arg_state_var):
        self.name = arg_name
        self.direction = arg_direction
        self.state_Var = arg_state_var


class Action:

    def __init__(self, service, name, arguments):
        # self.parent_service = service
        self.name = name
        self.arguments = arguments


class StateVariable:

    def __init__(self, service, name, send_events, data_type, values):
        # self.parent_service = service
        self.name = name
        self.send_events = send_events
        self.data_type = data_type
        self.allowed_values = values


class SubscribeRequest(object):

    def __init__(self, service, event_host, callback, cargo):
        log.debug("subscribe")
        self.callback = callback
        self.cargo = cargo
        self.service = service

        addr = service.event_sub_url
        Paddr = parse_url(addr)

        headers = {}
        headers["Host"] = Paddr.hostname
        headers["User-agent"] = 'BRISA-CP'
        headers["TIMEOUT"] = 'Second-300'
        headers["NT"] = 'upnp:event'
        headers["CALLBACK"] = "<http://%s:%d/eventSub>" % event_host
        headers["HOST"] = '%s:%d' % (Paddr.hostname, Paddr.port)

        run_async_call(http_call, success_callback=self.response,
                       error_callback=self.error, delay=0,
                       method='SUBSCRIBE', url=addr,
                       headers=headers)

    def error(self, error):
        log.debug("error", error)
        self.service.event_sid = ""
        self.service.event_timeout = 0
        if self.callback:
            self.callback(self.cargo, "", 0)
        return True

    def response(self, http_response):
        log.debug("response")
        compressed_headers = {}
        for k, v in dict(http_response.getheaders()).items():
            if not v:
                v = ""
            compressed_headers[k.lower()] = v.lower().strip()
            if 'sid' in compressed_headers:
                sid = compressed_headers['sid']
                timeout = 300
                if 'timeout' in compressed_headers:
                    stimeout = compressed_headers['timeout']
                    if stimeout[0:7] == "second-":
                        try:
                            timeout = int(stimeout[7:])
                        except ValueError:
                            pass
                self.service.event_sid = sid
                self.service.event_timeout = timeout
                if self.callback:
                    self.callback(self.cargo, sid, timeout)
        return True


class UnsubscribeRequest(object):

    def __init__(self, service, event_host, callback, cargo):
        self.old_sid = service.event_sid
        service.event_sid = ""
        service.event_timeout = 0

        self.callback = callback
        self.cargo = cargo
        self.service = service

        addr = service.event_sub_url
        Paddr = parse_url(addr)

        headers = {}
        headers["Host"] = Paddr.hostname
        headers["User-agent"] = 'BRISA-CP'
        headers["HOST"] = '%s:%d' % (Paddr.hostname, Paddr.port)
        headers["SID"] = self.old_sid

        run_async_call(http_call, success_callback=self.response,
                       error_callback=self.error, delay=0,
                       method='UNSUBSCRIBE', url=addr, headers=headers)

    def error(self, error):
        if self.callback:
            self.callback(self.cargo, "")
        return True

    def response(self, data):
        if self.callback:
            self.callback(self.cargo, self.old_sid)
        return True

    def event_subscribe(self, event_host, callback, cargo):
        if self.event_sid:
            # already registered
            return
        SubscribeRequest(self, event_host, callback, cargo)

    def event_unsubscribe(self, event_host, callback, cargo):
        if not self.event_sid:
            # not registered
            return
        UnsubscribeRequest(self, event_host, callback, cargo)


class Argument:

    def __init__(self, arg_name, arg_direction, arg_state_var):
        self.name = arg_name
        self.direction = arg_direction
        self.state_Var = arg_state_var


class Action:

    def __init__(self, service, name, arguments):
        # self.parent_service = service
        self.name = name
        self.arguments = arguments


class StateVariable:

    def __init__(self, service, name, send_events, data_type, values):
        # self.parent_service = service
        self.name = name
        self.send_events = send_events
        self.data_type = data_type
        self.allowed_values = values
