# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.

# Copyright 2005, Tim Potter <tpot@samba.org>
# Copyright 2006 John-Mark Gurney <gurney_j@resnet.uroegon.edu>
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import random
from brisa import log
from brisa.utils.network_listeners import UDPListener
from brisa.utils.network_senders import UDPTransport
from brisa.utils.python import LoopingCall
from brisa.upnp.upnp_defaults import UPnPDefaults

SSDP_ADDR = UPnPDefaults.SSDP_ADDR
SSDP_PORT = UPnPDefaults.SSDP_PORT

class SSDPServer(object):
    """A class implementing a SSDP server.

    The notify_received and search_received methods are called when the
    appropriate type of datagram is received by the server."""
    # not used yet
    stdheaders = [('Server', 'Twisted, UPnP/1.0, python-upnp'), ]
    elements = {}
    known_device = {}
    own = {}
    _callbacks = {}

    def __init__(self, server_name, xml_description_file, max_age=1800):
        # Should we share sockets here?
        self.UDPListener = UDPListener(SSDP_ADDR, SSDP_PORT,
                                       data_callback=self.datagram_received)
        self.UDPListener.start()
        self.transport = UDPTransport()
        self.server_name = server_name
        self.xml_description_file = xml_description_file
        self.max_age = max_age
        refresh_time = 0.8 * self.max_age
        self.renew_loop = LoopingCall(self.renew_notifications)
        self.renew_loop.start(refresh_time, now=True)

    def datagram_received(self, data, (host, port)):
        """Handle a received multicast datagram."""
        try:
            header, payload = data.split('\r\n\r\n')
        except ValueError, err:
            log.error('Error while receiving datagram packet')

#        log.debug('SSDPServer: datagram_received callback (%s, %s): %s\n' %
# (str(host), str(port), data))

        lines = header.split('\r\n')
        cmd = lines[0].split(' ')
        lines = map(lambda x: x.replace(': ', ':', 1), lines[1:])
        lines = filter(lambda x: len(x) > 0, lines)

        headers = [x.split(':', 1) for x in lines]
        headers = dict(map(lambda x: (x[0].lower(), x[1]), headers))

        if cmd[0] == 'M-SEARCH' and cmd[1] == '*':
            # SSDP discovery
            log.debug('SSDPServer: command is M-Search')
            self.discovery_request(headers, (host, port))
        elif cmd[0] == 'NOTIFY' and cmd[1] == '*':
            # SSDP presence
            self.notify_received(headers, (host, port))
        else:
            log.info('Unknown SSDP command %s', cmd)

    def discovery_request(self, headers, (host, port)):
        """Process a discovery request.

        The response must be sent to the address specified by (host, port)."""

#        log.debug('SSDPServer: discovery requested by %s:%s with headers %s' % (host, port, headers))
        rightKey = 0

        # Do we know about this service?
        if headers['st'] == 'ssdp:all':
            for i in self.own.keys():
                hcopy = dict(headers.iteritems())
                hcopy['st'] = self.own[i]['ST']
                self.discovery_request(hcopy, (host, port))
            return

        for key in self.own.keys():
            if self.own[key]['ST'].split()[0] == (headers['st']).split()[0]:
                rightKey = key
                break
        else:
            log.info('Discovery request ST %s not found', headers['st'])
            return

        if rightKey == 0:
            log.error('Unknown Error in Discovery request for %s',
                          headers['st'])
            return

        # Generate a response
        response = []
        response.append('HTTP/1.1 200 OK')
        append = response.append

        [append('%s: %s' % (k, v)) for k, v in self.own[rightKey].items()]
        response.extend(('', ''))
        delay = random.randint(0, int(headers['mx']))
        # Avoid using a timer with delay 0 :)
        if delay:
            self.transport.send_delayed(delay, '\r\n'.join(response), (host, port))
        else:
            self.transport.send_data('\r\n'.join(response), (host, port))
        log.debug('SSDPServer: discovery_request response sent to (%s, %d)' % (host, port))

    def notify_received(self, headers, (host, port)):
        """Process a presence announcement.

        We just remember the details of the SSDP service announced."""

        if headers['nts'] == 'ssdp:alive':
            if not self.is_known_device(headers['usn']):
                if 'cache-control' not in headers:
                    headers['cache-control'] = 'max-age=1800'
                self.register(headers['usn'],
                              headers['nt'],
                              headers['location'],
                              headers['server'],
                              headers['cache-control'])
        elif headers['nts'] == 'ssdp:byebye':
            if self.is_known_device(headers['usn']):
                self.unregister(headers['usn'])
        else:
            log.info('Unknown subtype %s for notification type %s',
                         headers['nts'], headers['nt'])

    def clear_device_list(self):
        self.known_device.clear()

    def register(self, usn, st, location, server, cache_control):
        """Register a service.

        Register a service or device that this SSDP server will
        respond to."""

        self.known_device[usn] = {'USN': usn, 'LOCATION': location, 'ST': st,
                                      'EXT': '', 'SERVER': server,
                                      'CACHE-CONTROL': cache_control}

        if st == 'upnp:rootdevice':
            self.callback("new_device_event", st, self.known_device[usn])

    def local_register(self, usn, st, location, server, cache_control):
        log.info('Registering locally %s (%s)', st, location)
        self.own[usn] = {'USN': usn, 'LOCATION': location, 'ST': st, 'EXT': '',
                         'SERVER': server, 'CACHE-CONTROL': cache_control}

        if st == 'upnp:rootdevice':
            self.callback("new_device_event", st, self.own[usn])

        self.do_notify(usn)

    def register_root_device(self, root_device):
        if root_device.is_root_device():
            #Dont have embedded devices
            device_id = root_device.udn
            device_type = root_device.device_type
            device_server = "BRisa Webserver UPnP/1.0 %s" % self.server_name
            device_location = "%s/%s" % (root_device.location,
                                         self.xml_description_file)
            age = 'max-age=%d' % self.max_age
            self.local_register('%s::upnp:rootdevice' % device_id,
                               'upnp:rootdevice', device_location,
                               device_server, age)
            self.local_register(device_id, device_id, device_location,
                               device_server, age)
            self.local_register('%s%s%s' % (device_id, "::", device_type),
                                device_type, device_location,
                                device_server, age)

            local_register = self.local_register
            (local_register('%s%s%s' % (device_id, "::",
                                        service.service_type),
                                        service.service_type, device_location,
                                        device_server, age) for service in
                                                          root_device.services)

#            [local_register('%s%s%s' % (device_id, "::",
#                                        service.service_type),
#                                        service.service_type, device_location,
#                                        device_server, age) for service in
#                                                          root_device.services]

        else:
            #has embedded devices
            #TODO: implement-me
            pass

    def announce_device(self, device):
        do_notify = self.do_notify
        (do_notify(usn) for usn in self.known_device)

    def renew_notifications(self):
        own_temp = self.own.copy()
        for usn in own_temp:
            log.debug('Renew notification for %s ', own_temp[usn]['USN'])
            self.do_notify(own_temp[usn]['USN'])

    def unregister(self, usn):
        log.info("Un-registering %s", usn)

        try:
            self.callback("removed_device_event", self.known_device[usn])
        except:
            pass

        del self.known_device[usn]

    def is_known_device(self, usn):
        return (usn in self.known_device)

    def subscribe(self, name, callback):
        self._callbacks.setdefault(name, []).append(callback)

    def unsubscribe(self, name, callback):
        callbacks = self._callbacks.get(name, [])
        if callback in callbacks:
            callbacks.remove(callback)
        self._callbacks[name] = callbacks

    def callback(self, name, *args):
        for callback in self._callbacks.get(name, []):
            callback(*args)

    def do_notify(self, st):
        """Do notification"""
        log.info('Sending alive notification for %s', st)
        resp = ['NOTIFY * HTTP/1.1', 'HOST: %s:%d' % (SSDP_ADDR, SSDP_PORT),
                'NTS: ssdp:alive', ]
        stcpy = dict(self.own[st].iteritems())
        stcpy['NT'] = stcpy['ST']
        del stcpy['EXT']
        del stcpy['ST']
        resp.extend(map(lambda x: ': '.join(x), stcpy.iteritems()))
        resp.extend(('', ''))
        self.transport.send_data('\r\n'.join(resp), (SSDP_ADDR, SSDP_PORT))
        self.transport.send_data('\r\n'.join(resp), (SSDP_ADDR, SSDP_PORT))

    def do_byebye(self, st):
        """Do byebye"""
        log.info('Sending byebye notification for %s', st)
        resp = ['NOTIFY * HTTP/1.1', 'HOST: %s:%d' % (SSDP_ADDR, SSDP_PORT),
                'NTS: ssdp:byebye', ]
        stcpy = dict(self.own[st].iteritems())
        stcpy['NT'] = stcpy['ST']
        del stcpy['ST']
        del stcpy['EXT']
        resp.extend(map(lambda x: ': '.join(x), stcpy.iteritems()))
        resp.extend(('', ''))
        log.debug(resp)
        self.transport.send_data('\r\n'.join(resp), (SSDP_ADDR, SSDP_PORT))
        self.transport.send_data('\r\n'.join(resp), (SSDP_ADDR, SSDP_PORT))

    def doStop(self):
        """Make sure we send out the byebye notifications."""
        own_temp = self.own.copy()
        for st in own_temp:
            self.do_byebye(st)
        log.info('Waiting for other threads to finish...')

        # Stop LoopingCall
        self.renew_loop.stop()

        # Stop listening
        self.UDPListener.stop()
