# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import urllib2
import httplib
import shutil
import socket
import fcntl
from time import time, sleep
from struct import pack
from urlparse import urlparse
from xml.etree import ElementTree

from brisa import log

socket.setdefaulttimeout(15)


def get_ip_address(ifname):
    """ Determine the IP address given the interface name

    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/439094
    (c) Paul Cannon
    Uses the Linux SIOCGIFADDR ioctl to find the IP address associated
    with a network interface, given the name of that interface, e.g. "eth0".
    The address is returned as a string containing a dotted quad.

    @param ifname: interface name
    @type ifname: string

    @return: ip address in the interface specified
    @rtype: string
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    ip = socket.inet_ntoa(fcntl.ioctl(s.fileno(), 0x8915,
                                      pack('256s', ifname[:15]))[20:24])
    log.debug("get ip address result: %s" % str(ip))
    return ip


def get_active_ifaces():
    """ Return a list of the active network interfaces

    Default route of /proc/net/route has the destination field set to 00000000

    @rtype: list
    """
    try:
        rd = open('/proc/net/route').readlines()
    except (IOError, OSError):
        return []
    net = [line.split('\t')[0:2] for line in rd]
    return [v[0] for v in net if v[1] == '00000000']


def http_call(method, url, body='', headers={}):
    """ Returns a HTTPResponse object for the given call.

    @param method: HTTP method (NOTIFY, POST, etc...)
    @param url: receiver URL
    @param body: body of the message
    @param headers: additional headers

    @type method: string
    @type url: string
    @type body: string
    @type headers: dictionary
    """
    parsed_url = urlparse(url)
    u = parsed_url[1]

    (host, ip, port) = ('', '', 80)

    if ':' in u:
        # host:port
        u = u.split(':')

        host = u[0]
        if len(u) == 2:
            port = int(u[1])

    if host:
        ip = socket.gethostbyname(host)
    else:
        log.debug('error: host is empty')

    log.debug('http call (host, port, ip): (%s, %d, %s)' % \
              (host, port, str(ip)))

    con = httplib.HTTPConnection("%s:%d" % (ip, port))
    con.connect()

    if body or headers:
        con.request(method, parsed_url[2], body=body, headers=headers)
    else:
        return None

    return con.getresponse()


def url_fetch(url, file='', attempts=2, interval=0.1):
    """ Fetches an URL.

    @param url: URL to be fetched
    @param file: if specified fetch result gets written in a file on this path
    @param attempts: number of attempts
    @param interval: interval between attempts in seconds

    @type url: string
    @type file: string
    @type attempts: integer
    @type interval: float
    """
    handle = None
    for k in range(attempts):
        log.info('[%d] Fetching %r (attempt %d)' % (time(), url, k))
        req = urllib2.Request(url)
        try:
            handle = urllib2.urlopen(req)
        except IOError, e:
            if hasattr(e, 'reason'):
                log.warning('[%d] We failed to reach a server. Reason: %s' %
                            (time(), e.reason))
            elif hasattr(e, 'code'):
                log.warning('[%d] The server couldn\'t fulfill the request. \
                            Error code: %s' % (time(), e.code))
            handle = None
        finally:
            if handle != None:
                if not file:
                    # Return mode
                    return handle
                else:
                    log.debug('url_fetch: writing data to file %s' % file)
                    # Writing mode
                    shutil.copyfile(handle, open(file, 'w'))
                    return None
            sleep(interval)
    return None


def decode(text):
    """Best-effort converting an arbitrary string to byte string in UTF-8"""

    if type(text) is unicode:
        return text.encode("utf-8")

    encoding_lst = [("iso-8859-15", ), ("utf-8", ), ("latin-1", ),
                    ("utf-8", "replace")]
    for encoding in encoding_lst:
        try:
            return text.decode(*encoding).encode("utf-8")
        except:
            pass
    return "???"   # Return something indicating we had problem w/ encoding


def parse_xml(data, encoding="utf-8"):
    p = ElementTree.XMLParser()
    p.feed(decode(data))
    return ElementTree.ElementTree(p.close())


def parse_http_response(data):
    header, payload = data.split('\r\n\r\n')
    lines = header.split('\r\n')
    cmd = lines[0].split(' ')
    lines = map(lambda x: x.replace(': ', ':', 1), lines[1:])
    lines = filter(lambda x: len(x) > 0, lines)
    headers = [x.split(':', 1) for x in lines]
    headers = dict(map(lambda x: (x[0].lower(), x[1]), headers))

    return cmd, headers


def parse_url(url):
    return urlparse(url)
