# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import socket
from time import time
from struct import pack

from brisa import log
from brisa.main import ThreadObject, ThreadManager

from brisa.upnp.upnp_defaults import UPnPDefaults


# UPnP default address and port (MCAST)
SSDP_ADDR = UPnPDefaults.SSDP_ADDR
SSDP_PORT = UPnPDefaults.SSDP_PORT


class CannotListenError(Exception):

    def __init__(self, interface, port, addr='', reason=''):
        if addr:
            self.message = "Couldn't listen on %s:%s: %d. " % (interface,
                                                             addr, port)
        else:
            self.message = "Couldn't listen on %s: %d. " % (interface,
                                                               port)
        if reason:
            self.message += reason


class SubscriptionError(Exception):
    message = "Couldn't subscribe listener because it does not implement \
    data_received. Implement brisa.utils.network_listeners.INetworkListener."


class INetworkListener:

    def data_received(self, data, addr=''):
        raise Exception("Classes implementing INetworkListener must implement \
                        data_received() method")


class NetworkListen(ThreadObject):
    """ Network listen abstract class. Forwards data to multiple subscribed
    listeners and can have a single callback to get data forwarded to.

    Methods that MUST be implemented by an inheriting class:
    - run()   : main loop that receives data. In order to forward data to
    listeners and the data callback run() method MUST call
    self.forward_data(data, addr). Note that addr is optional.
    - close() : stops connection
    """

    def __init__(self, listeners=[], data_callback=None):
        ThreadObject.__init__(self)
        self.listening = False
        self.data_callback = data_callback
        self.listeners = listeners

    def forward_data(self, data, addr=''):
        for listener in self.listeners:
            if addr:
                ThreadManager().run_async_function(listener.data_received,
                                                   (data, addr))
            else:
                ThreadManager().run_async_function(listener.data_received,
                                                   (data, ()))

        if self.data_callback:
            if addr:
                ThreadManager().run_async_function(self.data_callback,
                                                   (data, addr))
            else:
                ThreadManager().run_async_function(self.data_callback,
                                                   (data, ()))

    def subscribe(self, listener):
        if hasattr(listener, data_received) and listener not in self.listeners:
            self.listeners.append(listener)
        else:
            raise SubscriptionError()

    def is_listening(self):
        return self.listening

    def prepare_to_stop(self):
        # Overwrites ThreadObject's prepare_to_stop method
        if self.listening:
            self.listening = False
            self.close()

    def run(self):
        raise Exception("You must implement the run() method.")

    def close(self):
        raise Exception("You must implement the close() method for closing the\
        connection. For example, for a socket you should call socket.close().")


class TCPListener(NetworkListen):
    """ Listen TCP in a given port. Data received is forwarded to the
    listeners subscribed.

    """
    BUFFER_SIZE = 1500

    def __init__(self, port, interface='', listeners=[], data_callback=None,
                 shared_socket=None):
        NetworkListen.__init__(self, listeners, data_callback)
        self.port = port
        self.interface = interface
        self.socket = None
        self._create_socket(shared_socket)

    def _create_socket(self, shared):
        if shared:
            self.socket = shared
        else:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    def prepare_to_start(self):
        self._create_socket()

    def run(self):
        try:
            self.socket.bind((self.interface, self.port))
            self.listening = True
        except:
            raise CannotListenError(self.interface, self.port,
                                    reason="Couldn't bind address.")

        log.debug('TCPListener started')

        while self.listening:
            try:
                (data, addr) = self.socket.recvfrom(self.BUFFER_SIZE)
                self.forward_data(data, addr)
            except KeyboardInterrupt:
                log.debug('TCPListener shutdown')
                self.stop()
            except Exception, e:
                log.debug('TCPListener internal error, please report \
                         this error')

    def close(self):
        log.debug('Closing TCPListener socket')
        self.socket.close()


class UDPListener(NetworkListen):
    """ Listen UDP in a given address and port (if provided, in the given
    interface).
    """
    BUFFER_SIZE = 1500

    def __init__(self, addr=SSDP_ADDR, port=SSDP_PORT, interface='',
                 listeners=[], data_callback=None, shared_socket=None):
        NetworkListen.__init__(self, listeners, data_callback)
        self.addr = addr
        self.port = port
        self.interface = interface
        self.socket = None
        self._create_socket(shared_socket)

    def _create_socket(self, shared):
        if shared:
            self.socket = shared
        else:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM,
                                         socket.IPPROTO_UDP)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        try:
            self.socket.bind((self.interface, self.port))
            self.mreq = pack('4sl', socket.inet_aton(self.addr),
                             socket.INADDR_ANY)
            self.socket.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP,
                                    self.mreq)
        except:
            log.debug('UDPListener: CRITICAL ERROR - cannot create socket')
            raise CannotListenError(self.interface, self.port, self.addr,
                                    "Couldn't bind address")
        log.debug('UDPListener: socket created')

    def run(self):
        self.listening = True

        while self.listening:
            try:
                (data, addr) = self.socket.recvfrom(self.BUFFER_SIZE)
                self.forward_data(data, addr)
            except KeyboardInterrupt:
                log.debug('UDPListener: shutdown')
                self.stop()
            except Exception, e:
                log.debug('UDPListener: internal exception, %s' % e.message)

    def close(self):
        log.debug('UDPListener: closing socket')
        self.socket.close()

